package CPU.instructions.LEA;
import CPU.DisassembledOperand;
import CPU.InstructionSet;
import CPU.DisassembledInstruction;
import CPU.Size;

/**
 * The LEA instructions where source is .W absolute.
 * 
 * 0100dddd111mmmrrr
 * 
 * 
 * where dd = destination reg
 * mmm = source mode = 111
 * rrr = source reg =000
 * @author and Copyright (c) for my code Wolfgang Lenerz 2013
 * 
 * Based on code by Tony Headford, see his licence in accompanying file.
 * 
 */
public class LEA_W implements InstructionSet
{
    @Override
    public final void register(final CPU.MC68000Cpu cpu2)
    {
        int base = 0x41f8;
        
        CPU.Instruction i= new CPU.Instruction() 
        { 
            
                    
            @Override
            public final void execute(int opcode,CPU.MC68000Cpu cpu)
            {
                cpu.addr_regs[(opcode >> 9) & 0x07]=cpu.readMemoryWordPCSignedInc();         // get mem sign extended
            }
            @Override
            public DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
            {
                DisassembledOperand src = cpu.disassembleSrcEA(address + 2, (opcode >> 3) & 0x07, (opcode & 0x07), Size.Long);
                DisassembledOperand dst = new DisassembledOperand("a" + ((opcode >> 9) & 0x07));
                return new DisassembledInstruction(address, opcode, "lea", src, dst);
            }
        };
        for(int r = 0; r < 8; r++)
        {
                cpu2.addInstruction(base + (r << 9) , i);
        }
    }
}
