package CPU.instructions;
import CPU.DisassembledOperand;
import CPU.InstructionSet;
import CPU.DisassembledInstruction;
import CPU.Size;

/**
 * The PEA instruction in all of its variants.
 * 
 *  0100100001mmmrrr =0x4840
 * where    rrr = src reg
 *          mmm = mod
 *       
 * @author and Copyright (c) for my code Wolfgang Lenerz 2013
 * 
 * Based on code by Tony Headford, see his licence in accompanying file.
 */
public class PEA implements InstructionSet
{
  
    @Override
    public final void register(final CPU.MC68000Cpu cpu2)
    {
        CPU.Instruction i = new CPU.Instruction() 
        {
            @Override
            public final void execute(int opcode,CPU.MC68000Cpu cpu)
            {
                int address=0;
                switch ((opcode >>3)&7)
                {
                   
                    case 2:                                     //(an)
                        address=cpu.addr_regs[opcode&7];
                        break;
                    case 5:                                     // d16(an)
                        address=cpu.addr_regs[opcode&7] + cpu.readMemoryWordPCSignedInc();
                        break;
                    case 6:                                     // d8(an,xn)
                        address=getDisplacement(cpu)+cpu.addr_regs[opcode&7];
                        break;
                    case 7:
                        switch (opcode &7)
                        {
                            case 0:
                                address=cpu.readMemoryWordPCSignedInc();
                                break;
                            case 1:
                                address=cpu.readMemoryLongPC();
                                cpu.pc_reg+=2;
                                break;
                            case 2:
                                address=cpu.pc_reg*2 + cpu.readMemoryWordPCSignedInc();
                                break;
                            case 3:
                                address= cpu.pc_reg*2 + getDisplacement(cpu);
                                break;
                        }
                        break;
                }
                cpu.addr_regs[7]-=4;
                cpu.writeMemoryLong(cpu.addr_regs[7], address);
            }
            
            @Override
            public DisassembledInstruction disassemble(int address, int opcode,CPU.MC68000Cpu cpu)
            {
                DisassembledOperand src = cpu.disassembleSrcEA(address + 2, (opcode >> 3) & 0x07, (opcode & 0x07), Size.Long);
		return new DisassembledInstruction(address, opcode, "pea", src);
            } 
            
            protected int getDisplacement(CPU.MC68000Cpu cpu)
            {
                int ext = cpu.readMemoryWordPCSignedInc();// extention word, contains size + displacement+reg
                int displacement=(ext & 0x80)!=0?ext| 0xffffff00:ext&0xff;    //displacemnt
                if((ext & 0x8000) !=0)
                {
                    if((ext & 0x0800) == 0)                // word or long register displacement?
                    {
                        displacement+= cpu.signExtendWord(cpu.addr_regs[(ext >> 12) & 0x07]);
                    }
                    else
                    {
                        displacement+= cpu.addr_regs[(ext >> 12) & 0x07];
                    }
                }
                else
                {
                    if((ext & 0x0800) == 0)                // word or long register displacement?
                    {
                        displacement+= cpu.signExtendWord(cpu.data_regs[(ext >> 12) & 0x07]);
                    }
                    else
                    {
                        displacement+= cpu.data_regs[(ext >> 12) & 0x07];
                    }
                }
                return displacement;                                // don't modify reg_pc here (case 7-3)
            }
            
        };
        int base=0x4840;
        for (int mode=2;mode<8;mode++)
        {
            if (mode==3 || mode==4)
                continue;
            int rmode=mode<<3;
            for (int reg=0;reg<8;reg++)
            {
                if (mode==7 & reg>3)
                    break;
                cpu2.addInstruction(base+ rmode + reg, i);
            }
        }
    }
}
