package smsqmulator;

/**
 * The class that does the "simple" (fast) emulation.
 * 
 * @author and copyright (c) Wolfgang Lenerz 2012-2023. 
 */
public class SimpleEmulationThread  extends Thread implements EmulationThread
{
    private final CPU.MC68000Cpu cpu;
    private QL50HzInterrupt ih;
    
    public SimpleEmulationThread(CPU.MC68000Cpu cpu, screens.Screen screen)
    {
        this.cpu=cpu;
        this.ih = new QL50HzInterrupt(cpu);
        this.ih.setScreen(screen);
    }    
    
    /**
     * Does the actual work : execute cpu instructions in a loop.
     */
    @Override
    public void run()
    {
        this.ih.startInterrupts();
        this.cpu.executeContinuous();                   // goes into continuous loop in the cpu object, doesn't come back here (unless problem)
    }
    
    /**
     *
     */
    @Override
    public void endEmulation()
    {
       if (this.ih!=null)
        {
            this.ih.stopInterrupts();
            this.ih.setScreen(null);
            this.ih=null;
        }
        this.stop();
    }
    
      
    /**
     * Sets the interval between screen updates.
     * 
     * @param tim the interval between screen updates in milliseconds.
     */
    @Override
    public void setScreenUpdateInterval(int tim)
    {
        if (this.ih!=null)
            this.ih.setScreenUpdateInterval(tim);
    }

    @Override
    public void switchLogging(boolean logInstructions)
    {
        throw new UnsupportedOperationException("Not supported yet."); // Generated from nbfs://nbhost/SystemFileSystem/Templates/Classes/Code/GeneratedMethodBody
    }

    @Override
    public void setckmem(boolean checkmem)
    {
        throw new UnsupportedOperationException("Not supported yet."); // Generated from nbfs://nbhost/SystemFileSystem/Templates/Classes/Code/GeneratedMethodBody
    }

    @Override
    public void setmemWatched(int address)
    {
        throw new UnsupportedOperationException("Not supported yet."); // Generated from nbfs://nbhost/SystemFileSystem/Templates/Classes/Code/GeneratedMethodBody
    }

    @Override
    public void setUpperLimit(int address)
    {
        throw new UnsupportedOperationException("Not supported yet."); // Generated from nbfs://nbhost/SystemFileSystem/Templates/Classes/Code/GeneratedMethodBody
    }

    @Override
    public void setCondition(int dataReg, int addrReg, int condValue, boolean isContent, int address)
    {
        throw new UnsupportedOperationException("Not supported yet."); // Generated from nbfs://nbhost/SystemFileSystem/Templates/Classes/Code/GeneratedMethodBody
    }
}
