/*
 * Copyright (c) W. Lenerz 2023.
 * This code is subject to the GPL, version 3 or later.
 */
package ui;

import smsqmulator.TrapDispatcher;

/**
 * The config menu in the menu bar, including item action routines.
 * 
 * @author and copyright Wolfgang Lenerz
 * @version  1.00 initial version
 */
public class ConfigMenu extends javax.swing.JMenu
{
    private final smsqmulator.Monitor monitor;
    private final inifile.IniFile inifile;
    private final MonitorGui mgui;
    private final Warnings warnings;
    
    /**
     * The CONFIG menu item in the menu bar.
     * 
     * @param pmonitor  monitor for the emultion thread
     * @param pinifile  the inifile with all configured options.
     * @param mguip     the monitorgui enclosing th menu bar.
     * @param pwarnings warnings
     */
    public ConfigMenu(smsqmulator.Monitor pmonitor,inifile.IniFile pinifile,MonitorGui mguip, Warnings pwarnings,TrapDispatcher td)
    {
        this.monitor=pmonitor;
        this.inifile=pinifile;
        this.mgui=mguip;
        this.warnings=pwarnings;
        initMenu(inifile, td);
    }
    private void initMenu(inifile.IniFile inifile,TrapDispatcher td)
    {
        javax.swing.JMenu menu;
        javax.swing.JCheckBoxMenuItem cbox;
        javax.swing.JMenuItem mit ;
        javax.swing.ButtonGroup buttonGroup;
        javax.swing.JRadioButtonMenuItem rbutton;
//  String s =    Localization.Texts[3];
  //s=s+"p";
        setText(Localization.Texts[3]);
        
        String[]filenames= {"images/flag-germany.gif","images/flag-uk.gif","images/flag-spain.gif","images/flag-france.gif","images/flag-italy.png"};
        int lang = this.inifile.getOptionAsInt("LANGUAGE",0);
        this.addLanguageButtons(new String [] {"Deutsch","English","Espanol","Français","Italiano"},inifile,filenames,lang);
        this.add(new javax.swing.JPopupMenu.Separator());
        this.add(new DevicesMenu(this.monitor,this.inifile,this.mgui,td));
        this.add(new javax.swing.JPopupMenu.Separator());
        this.add(new ScreenMenu(this.monitor,this.inifile,this.mgui));
        this.add(new javax.swing.JPopupMenu.Separator());
// sound
        mit= new javax.swing.JMenuItem();  
        mit.setText(Localization.Texts[113]);          
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            setBeepVolume();
        });
        this.add(mit);
        
        int freq = this.inifile.getOptionAsInt("SSSS-FREQUENCY", 22);
        buttonGroup = new javax.swing.ButtonGroup ();
        menu = new javax.swing.JMenu();
        menu.setText(Localization.Texts[143]);  
            rbutton = new javax.swing.JRadioButtonMenuItem ();         // set SampledSound frequency to 22.05khz.
            rbutton.setText("22.05 KHz (22050 Hz)");
            rbutton.addActionListener((java.awt.event.ActionEvent evt) ->
            {
                ConfigData.setOptionAndSave("SSSS-FREQUENCY","22",this.inifile);
            });
            if (freq==22)
                rbutton.setSelected(true);
            buttonGroup.add(rbutton);
            menu.add(rbutton );
            rbutton = new javax.swing.JRadioButtonMenuItem ();         // set SampledSound frequency to 20khz.
            rbutton.setText("22 KHz (22000 Hz)");
            if (freq==20)
                rbutton.setSelected(true);
            buttonGroup.add(rbutton);
            rbutton.addActionListener((java.awt.event.ActionEvent evt) ->
            {
                ConfigData.setOptionAndSave("SSSS-FREQUENCY","20",this.inifile);
            });
            menu.add(rbutton);
        this.add(menu);
        this.add(new javax.swing.JPopupMenu.Separator());
// memory 
        mit= new javax.swing.JMenuItem();                       // set memory size  
        mit.setText(Localization.Texts[18]);          
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            setMemSize();
        });
        this.add(mit);
        this.add(new javax.swing.JPopupMenu.Separator());
// debugging       
        cbox = new javax.swing.JCheckBoxMenuItem();             // monitor visible?
        cbox.setText(Localization.Texts[19]);
        cbox.setSelected(this.inifile.getTrueOrFalse("MONITOR-VISIBLE"));
        cbox.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            javax.swing.JCheckBoxMenuItem cb = (javax.swing.JCheckBoxMenuItem) evt.getSource();
            this.mgui.monitorVisible(cb.isSelected());
        });
        this.add(cbox);
        
        cbox = new javax.swing.JCheckBoxMenuItem();             // fast mode??
        cbox.setText(Localization.Texts[20]);
        cbox.setSelected(this.inifile.getTrueOrFalse("FAST-MODE"));
        cbox.addActionListener((java.awt.event.ActionEvent evt) ->
        {    
            setYesOrNo(evt,"FAST-MODE");
            this.monitor.setFastMode(); 
        });
        this.add(cbox);
        this.add(new javax.swing.JPopupMenu.Separator());
// cpu       
        cbox = new javax.swing.JCheckBoxMenuItem();             // sleep if idle?
        cbox.setText(Localization.Texts[121]);
        cbox.setSelected(this.inifile.getTrueOrFalse("LESS-CPU-WHEN-IDLE"));
        cbox.addActionListener((java.awt.event.ActionEvent evt) ->
        {   
            setYesOrNo(evt,"LESS-CPU-WHEN-IDLE");
        });
        this.add(cbox);
        
        cbox = new javax.swing.JCheckBoxMenuItem();                     // suspend when iconifier?
        cbox.setText(Localization.Texts[110]);          
        cbox.addActionListener((java.awt.event.ActionEvent evt) ->
        {
             setYesOrNo(evt,"SUSPEND-WHEN-ICONIFIED");
        });
        this.add(cbox);
         
        buttonGroup = new javax.swing.ButtonGroup ();
        menu = new javax.swing.JMenu();                                 // what to do when popup action?
        menu.setText(Localization.Texts[145]);  
            rbutton = new javax.swing.JRadioButtonMenuItem ();          // open wdw
            rbutton.setText(Localization.Texts[146]);
            rbutton.setSelected(!this.inifile.getTrueOrFalse("POPUP-ACTION"));
            rbutton.addActionListener((java.awt.event.ActionEvent evt) ->
            {
                ConfigData.setOptionAndSave("POPUP-ACTION","0",this.inifile);
            });
            buttonGroup.add(rbutton);
            menu.add(rbutton );
            rbutton = new javax.swing.JRadioButtonMenuItem ();         // blink button
            rbutton.setText(Localization.Texts[147]);
            rbutton.setSelected(this.inifile.getTrueOrFalse("POPUP-ACTION"));
            buttonGroup.add(rbutton);
            rbutton.addActionListener((java.awt.event.ActionEvent evt) ->
            {
                ConfigData.setOptionAndSave("POPUP-ACTION","1",this.inifile);
            });
            menu.add(rbutton);
        this.add(menu);
        this.add(new javax.swing.JPopupMenu.Separator());
 //  mouse
        mit= new javax.swing.JMenuItem();                       // mouse click delay 
        mit.setText(Localization.Texts[127]+this.inifile.getOptionAsInt("MOUSE-CLICK-DELAY",0));          
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            javax.swing.JMenuItem mitem = (javax.swing.JMenuItem) evt.getSource();
            setMouseClickDelay(mitem);
        });
        this.add(mit);                   
        
        mit= new javax.swing.JMenuItem();                       // mouse wheel "acceleration"
        mit.setText(Localization.Texts[148]+": "+this.inifile.getOptionAsInt("MOUSEWHEEL-ACCEL",0));          
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            javax.swing.JMenuItem mitem = (javax.swing.JMenuItem) evt.getSource();
            setMouseWheelAccel(mitem);
        });
        this.add(mit);
        this.add(new javax.swing.JPopupMenu.Separator());
// misc        
        mit= new javax.swing.JMenuItem();                       // time correction
        mit.setText(Localization.Texts[93]);          
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            setTimeOffset();
        });
        this.add(mit);   
        
        mit= new javax.swing.JMenuItem();                       // hide menu bar
        mit.setText(Localization.Texts[71]);          
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            this.mgui.setMenuBarVisible(false);
        });
        this.add(mit);   
        
        cbox = new javax.swing.JCheckBoxMenuItem();             // use standard home keys?
        cbox.setText(Localization.Texts[154]);
        cbox.setSelected(this.inifile.getTrueOrFalse("STANDARD_HOME_END_KEYS"));
        cbox.addActionListener((java.awt.event.ActionEvent evt) ->
        {     
            javax.swing.JCheckBoxMenuItem cb = (javax.swing.JCheckBoxMenuItem) evt.getSource();
            this.mgui.getScreen().setKeysType(cb.isSelected());
            ConfigData.setOptionAndSave("STANDARD_HOME_END_KEYS", cb.isSelected()? "1":"0",this.inifile);
        });
        this.add(cbox);                
        this.add(new javax.swing.JPopupMenu.Separator());
// warnings                
        this.add(new WarningsMenu(this.inifile,this.mgui,this.warnings));
    } 
                          
    private void setLanguage(javax.swing.JRadioButtonMenuItem item)
    {
        int nbr = 0;
        try
        {
            nbr = Integer.parseInt(item.getActionCommand());
        }
        catch (Exception e)
        {
            return;
        }
        ConfigData.setOptionAndSave("LANGUAGE",""+nbr,inifile);
        this.mgui.setLanguage(nbr);;
    }
    
    private void addLanguageButtons(String [] texts,final inifile.IniFile inifile,String[]imageFileNames, int presetLanguage)
    {
        javax.swing.JMenu languageMenu = new javax.swing.JMenu();
        languageMenu.setText("Language/Sprache/Lengua/Langue/Lingua");
        javax.swing.ButtonGroup buttonGroup = new javax.swing.ButtonGroup();
        javax.swing.JRadioButtonMenuItem item;
        for (int i = 0;i<texts.length; i++)
        {
            item = new javax.swing.JRadioButtonMenuItem();
            buttonGroup.add(item) ;
            item.setText(texts[i]);
            item.setActionCommand(""+(i+1));
            if ((i+1)==presetLanguage)
                item.setSelected(true);
            item.addActionListener((java.awt.event.ActionEvent evt) ->
            {
                javax.swing.JRadioButtonMenuItem rb = (javax.swing.JRadioButtonMenuItem) evt.getSource();
                int nbr;
                try
                {
                    nbr = Integer.parseInt(rb.getActionCommand());
                }
                catch (NumberFormatException e)
                {
                    return;
                }
                ConfigData.setOptionAndSave("LANGUAGE",""+nbr,inifile);
                this.mgui.setLanguage(nbr);
            });
            item.setIcon(new javax.swing.ImageIcon(getClass().getResource(imageFileNames[i])));
            languageMenu.add(item);
        }
        this.add(languageMenu);
    }
    
    /**
     * Menu item action : Change the size of the memory to be used.
     * 
     *<p> This implies a change of the CPU object!
     */
    private void setMemSize()
    {                                                
        screens.Screen screen = this.mgui.getScreen();
        int mode=screen!=null ? screen.getColourMode() : 0; 
        Object[]  memPossibilities = {1,2,4,8,16,32,64,96,128,256,384,480,500,512};// the memory possibilities we have
        Object[]  possibilities=new Object[memPossibilities.length-3]; // memory sizes proposed to the user depending on screen mode
        if (mode>8)
        {
           System.arraycopy(memPossibilities,3,possibilities,0,memPossibilities.length-3);// less choice if higher colours, we need more mem
        }
        else                                                    // note i'm just too lazy to instantiate the array twice
        {
            possibilities=memPossibilities;                     // in lower screens, can use less mem
        }
        int memSize=this.monitor.getCPU().getRamSizeInMiB();
        Object s= javax.swing.JOptionPane.showInputDialog(this, Localization.Texts[26]+"\n"+Localization.Texts[27]+
                memSize+"MB\n"+"<html><font color=red>"+Localization.Texts[29]+"</font></html>",
                Localization.Texts[28],javax.swing.JOptionPane.PLAIN_MESSAGE, null, possibilities, memSize);// get mem size now
       if (s!=null)
        {
            try
            { 
                memSize=(Integer)s;
                ConfigData.setOptionAndSave("MEM_SIZE", ""+memSize,this.inifile);                
                this.monitor.reset();
           //     this.monitor.changeMemSize(-1,screen,this.inifile.getTrueOrFalse("QL-SCREEN-EMULATION"));
            }
            catch (Exception e)
            {
                javax.swing.JOptionPane.showMessageDialog(this, Localization.Texts[30], s+Localization.Texts[31],javax.swing.JOptionPane.ERROR_MESSAGE);
            }
        }
    }       

    /**
     * Menu item action : Sets the volume of the beep sound.
     
     * @param evt ignored
     */
    private void setBeepVolume() 
    {
        int vol = GetIntegerInput.getIntegerInput(this, this.inifile.getOptionAsInt("SOUND-VOLUME",0) , 
                    Localization.Texts[112],Localization.Texts[113], 100);
        if (vol>=0)
        {
            ConfigData.setOptionAndSave("SOUND-VOLUME",""+vol,this.inifile);
            this.monitor.getTrapDispatcher().setSoundVolume (vol);
        }
    }
    
    /**
     * Menu item action : Sets the delay when a mouse button is clicked.
     * 
     */
    private void setMouseClickDelay(javax.swing.JMenuItem item)                                                        
    {        
        int delay = GetIntegerInput.getIntegerInput(this, this.inifile.getOptionAsInt("MOUSE-CLICK-DELAY",0) , 
                        Localization.Texts[128],Localization.Texts[127], -1);
        if (delay >=0)
        {
            ConfigData.setOptionAndSave("MOUSE-CLICK-DELAY",""+delay,this.inifile);
            item.setText(Localization.Texts[127]+delay);
            this.mgui.getScreen().setMouseClickDelay(delay);
        }
    }         
    
    /**
     * Menu item action : Sets the delay when a mouse button is clicked.
     * 
     */
    private void setMouseWheelAccel(javax.swing.JMenuItem item)                                                        
    {        
        int accel = GetIntegerInput.getIntegerInput(this, this.inifile.getOptionAsInt("MOUSEWHEEL-ACCEL",0) , 
                          Localization.Texts[148]+ " (0 - 9)", Localization.Texts[148], -1);
        if (accel >=0)
        { 
            item.setText(Localization.Texts[148]+ ": "+accel);
            ConfigData.setOptionAndSave("MOUSEWHEEL-ACCEL",""+accel,this.inifile);
            this.mgui.getScreen().setMousewheelAccel(accel);
        }
    }         
    
    /**
     * Menu item action : Sets the delay when a mouse button is clicked.
     * 
     */
    private void setTimeOffset()                                                        
    {        
        int offset = GetIntegerInput.getIntegerInput(this, this.inifile.getOptionAsInt("TIME-OFFSET",0) , 
                         Localization.Texts[94],Localization.Texts[93], -1);
        if (offset >=0)
        {
            ConfigData.setOptionAndSave("TIME-OFFSET",""+offset,this.inifile);
            this.monitor.getTrapDispatcher().setTimeOffset (offset);
        }
    }
   
    /**
     * Sets yes or no in ini file.
     * 
     * @param evt           event the source of which is a JCheckBoxMenuItem
     * @param optionName    name of option in config file
     */
    private void setYesOrNo(java.awt.event.ActionEvent evt, String optionName)
    {
         javax.swing.JCheckBoxMenuItem cb = (javax.swing.JCheckBoxMenuItem) evt.getSource();
         ConfigData.setOptionAndSave(optionName, cb.isSelected()? "1":"0",this.inifile);
    }
}