
package ui;

/**
 * This is the class responsible for handling some Mac specific code for the User Interface.
 * 
 * Much of this code (for java versions before 9) contributed by Tobias Fröschle.
 * 
 * @author and copyright (for my code) (C) wolfgang Lenerz 2012-2021
 */
public class MacUI
{
    /** try to set the mac LAF, 
     * 
     * @param version the major version of java. As of version 9, the apple extensions are no longer used!
     * @param gui 
     */
    public static final void setLAF(int version,MonitorGui gui)
    {
        java.net.URL imgURL = ui.MonitorGui.class.getResource("images/WolfSoft2.gif");
        java.awt.Image image = null;
        try 
        {
            image = javax.imageio.ImageIO.read(imgURL);
        } 
        catch (java.io.IOException ex) 
        {
            //NOP. If there is no image, then I'll have to do without.
        }
        if (version < 9)
        {                                      // code contributed by Tobias Fröschle
/*
            // do some Mac specific stuff like
            // 1. get the Mac application
            com.apple.eawt.Application app = com.apple.eawt.Application.getApplication();
            // 2. Set the dock image for the app
            if (image!=null)
                app.setDockIconImage(image);
            
            // 3. Set a specific <CMD>+"Q" handler. We don't want the application 
            // to end if CMD-Q is pressed inadvertantly.
            app.setQuitHandler(new com.apple.eawt.QuitHandler() 
            {
                @Override
                public void handleQuitRequestWith(com.apple.eawt.AppEvent.QuitEvent qe, com.apple.eawt.QuitResponse qr) 
                {
                    // Intention would be that when activated from the menu, then we should quit, 
                    // when activated through <CMD>-Q, inject the keypress but that's not quite soooo simple.....
                    if (qe.getSource() != gui)
                        qr.cancelQuit();
                    else
                        qr.performQuit();
                }
            });
    */      
        }
        else
        {
 
            try
            {
                final java.awt.Taskbar taskbar =  java.awt.Taskbar.getTaskbar();
                taskbar.setIconImage(image);
                
                java.awt.Desktop desktop =  java.awt.Desktop.getDesktop();
                QuitChecker qc = new QuitChecker(gui);
                desktop.setQuitHandler(qc);
            }
            catch (Exception e)
            {
                // just give up - NOP
            }  
      
        }

        // Set some properties to be a bit more Apple Application Guidelines conformant
        // Screen menu bar instead of window menu bar (also saves a bit of screen real estate)
        System.setProperty("apple.laf.useScreenMenuBar", "true");
        System.setProperty("apple.awt.brushMetalLook", "true");
        
    }
    
    public static final void setFullScreen(MonitorGui gui,java.awt.GraphicsDevice device, int version)
    {
        if (version<9)  
        {
       
            try
            {
                com.apple.eawt.FullScreenUtilities.setWindowCanFullScreen(gui,true);
                com.apple.eawt.Application.getApplication().requestToggleFullScreen(gui);
            }
            catch (Exception ex)
            {
                gui.setExtendedState(javax.swing.JFrame.MAXIMIZED_BOTH); 
                device.setFullScreenWindow(gui);
            }
            
        }
        else
        {
            gui.setExtendedState(javax.swing.JFrame.MAXIMIZED_BOTH); 
            device.setFullScreenWindow(gui);
        }
    }

    private static class QuitChecker implements java.awt.desktop.QuitHandler
    {
        private final MonitorGui mon;
        private QuitChecker (final MonitorGui moni)
        {
            this.mon=moni;
        }
        
        @Override
        public void handleQuitRequestWith(java.awt.desktop.QuitEvent qe, java.awt.desktop.QuitResponse qr)
        {
            if (qe.getSource() != this.mon)
                qr.cancelQuit();
            else
                qr.performQuit();
        }
    }

}
