package ui;

import smsqmulator.Monitor;
import screens.Screen;
import smsqmulator.TrapDispatcher;

/**
 * The GUI for the monitor/emulator.
 * 
 * Handles the User interface.
 * 
 * @author and copyright (c) Wolfgang Lenerz 2012-2023.
 * 
 * @version 
 * 2.00 ser ports assignment dialog integrated + action routine, all menus were split off, setlanguage is done by recreating all menus,  many actions delegated to other components.
 * 1.39 FLP drives are stored in inifile again, after being changed (probably broken in 1.38).
 * 1.38 Use ResolvePath to resolve special paths, change the way paths are held/displayed, setting floppy names displays the required dialog again.
 * 1.37 Mac specific code moved to MacUI.
 * 1.36 set correct useXTcc status in checkbox, use status immediately in NFA after change in menu.
 * 1.35 add HDD in setNewDeviceNames, set mouse position when de-iconifying, delete keystroke and mouse click when iconifying, robot sets mouse pos when changed from SMSQ/E (formerly in TrapDispatcher)
 * 1.34 Creation params modified (removed vibrantColours as inifile is also passed), some instance vars removed (memsize). FormIconified introduces delay for windows;
 *      use BUTTON1DOWN_MASK instead of BUTTON1_MASK (doAltSpace).
 * 1.33 when setting device names, show to what "./" will be expanded
 * 1.32 setDeviceNames : when setting the ptr. pos. for opening the dad wdw, use a new point if ptr is outsie the wdw.
 * 1.31 device names set from within SMSQ/E are stored in the inifile ; new config item locked 
 *      qxl.win files may be read only ; SUSPEND-WHEN-ICONIFIED state is actually loaded at startup ;
 *      new config item action upon jva_popup ; ssssFrequencyMenuItem text is set accoring to language ; java version item added
 * 1.30 allowScreenEmulation config item added.
 * 1.29 Exit item in files menu and action routine ; provide for full screen modes:
 *      creation requires new parameters (window mode) ; if full screen window mode, wdw is undecorated ; call screen.setFullSize ; 
 *      new config items : window mode and display full screen on which monitor?, and corresponding action routines.
 * 1.28 Monitor.changeMemSize amended parameter used when changing mem size, screenSizeMenuItemActionPerformed makes new screen,
 *      screenMode variable introduced, monitor.reset() no longEr takes params, when screen mode/colours are changed,
 *      this takes effect immediately (setScreenModeForNextRestart), Throttle removed.
 * 1.27 getMonitor() implemented, screen update rate, devices sub-menu added.
 * 1.26 optimizations, italian language.
 * 1.25 use java.awt.event.InputEvent.BUTTON1_MASK instead of java.awt.event.InputEvent.BUTTON1_DOWN_MASK in doAltSpace,
 *      don't print ALT F3 to system.out
 * 1.24 new config item for mouse click delay and related action routine.
 * 1.23 new config item for qxl.win files lock error.
 * 1.22 only use alt-space kludge for windows, better ALT+SPACE handling when in windows. The systems menu is closed and the space key is marked as released!
 * 1.21 warning for read-only floppy drives.
 * 1.20 only call getOptions once wdw is open - shows errors in wdw ....
 * 1.19 new config menu item (use less CPU time when idle) ; made group of everything relating to energy/cpu saving
 * 1.18 this object is now split between this remaining object and the Screen object : routines relating to mouse/key presses moved to the screen object;
 *      use MonitorPanel object instead of creating these panels myself ; formWindowOpened starts the emulation, not the Monitor object when loading ROM.
 * 1.17 "warn if sound problem" config item and supporting code added.
 * 1.16 xsize must be multiple of 8 for all screen modes.
 * 1.15 cleanup (make sure screen object is removed when new screen is made). Screen size may be given as a * b instead of a x b.
 *      Slightly more bright colours menu item added. Set windowActivated status, ptr in wdw is only set when ptr "in screen" and wdw is activated.
 *      Provide for checkbox to suspend execution if window is minimized and call monitor.
 * 1.14 provide for 8 bit colours choice ; save rom file name before raising error.
 * 1.13 handle alt + cursor keys ; mouse wheel generates three keystrokes ; about shows java version for which this was compiled ; 
 *          when opening wdw, reposition wdw at its last position.
 * 1.12 set "pointer in screen" status; use new icon; changed some "config" menu item positions.
 * 1.11 correctly set status of read only warning item on startup; handle Shift tab, shift del, alt del, shift alt del. 
 * 1.10 better mouse handling when in double sized mode, warning for read only qxl.win drives added, set screen coordinates when wdw moved/resized ;
 *      ignore mouse moved event when mouse position in wdw is changed from within SMSQE via sptr.
 * 1.09 in memsize choice box current memsize is the default value. Max mem size = 96 MiB
 * 1.08 make sure screen is at least 512 x 256 pixels.
 * 1.07 when screen size is set, record true screen size in .ini file (could have been changed if in mode 4).
 *      Caps lock status is set in SMSQ/E. Show the SMSQ/E screen size when changing screen size, not the window size.
 *      Better handling of doubling/halving screen size.
 * 1.06 correct check whether screen was in double size upon launching or not. Warning items status set in ini file again.
 *      menu bar can be made invisible (use JVAMBAR Sbasic command to get it back).
 * 1.05 the time correction item really calls the time correction code.
 * 1.04 screen size (but not resolution) changes when the window is resized. Possibility to double the window size (not resolution) introduced.
 * 1.03 warn that machine will be reset when screen size is changed.
 * 1.02 provide for "time correction" option.
 * 1.01 handle some accented chars,"wake up" monitorGo thread when key typed.
 * 1.00 set warning checkboxes status on startup, added Throttle config item,only make new (QXL,SFA,NFA,FLP) filenames if OK was hit in DAD wdw.
 * 0.05 added some Mac specific code for keyboard handling and window appearance, courtesy of Tobias Fröschle.
 * 0.04 device usage configuration item : it set to "", device name is set back to original name.
 * 0.03 F12 = alt+space, first attempts at mac keyboard handling.
 * 0.02 many more configurable items.
 * 0.01 traps mouse movement and buttons.
 * 0.00 initial version
 */
public class MonitorGui extends javax.swing.JFrame
{
    private final Monitor monitor;
    private final inifile.IniFile inifile;                  // the .ini file with saved options
    private Screen screen;                                  // the screen object for the emulator - this is also a swing.JPanel.
    private boolean windowActivated=false;
    private final MonitorPanel monitorPanel;                // panel for monitoring ops
    private java.awt.Robot robot;                           // cannot be made final
    private int windowMode;                                 // mode of swing wdw: 0 =window, 1 = full size, 2 = special full size
    private java.awt.GraphicsDevice[] gds;                  // the monitors in a multi monitor environment
    
    /**
     * Creates new form MonitorGui.
     * 
     * @param inifile the <code>Inifile</code> object containing configured values.
     * @param gc the graphics configuration of the current monitor screen we're on.
     * @param gds the info about all monitors that the user has connected.
     * @param monit monitor used
     * 
     */
    public MonitorGui(inifile.IniFile inifile, java.awt.GraphicsConfiguration gc, java.awt.GraphicsDevice[] gds, Monitor monit,TrapDispatcher td)
    {
        super (gc);
        this.windowMode=inifile.getOptionAsInt("WINDOW-MODE", 0);
        this.inifile=inifile;
        this.gds=gds;
        this.monitor=monit;
        
        String os=System.getProperty("os.name").toLowerCase();
        boolean isMac = os.contains("mac");
        boolean isWindows = os.contains("windows");
        if (isMac)  
            MacUI.setLAF(11,this);
        
        this.windowMode = inifile.getOptionAsInt("WINDOW-MODE", 0);
        if (this.windowMode>2 || this.windowMode<0)
            this.windowMode=0;                    
        
        int xsize=inifile.getOptionAsInt("WDW_XSIZE", 512);
        int ysize=inifile.getOptionAsInt("WDW_YSIZE", 256);
        if (xsize<512)
            xsize=512;
        if (ysize<256)
            ysize=256;                                          // 512*256 = minimum x,y sizes
        xsize += (xsize % 8)==0?  0 : (8-(xsize % 8));          // make x size a multiple of 8
       
        java.awt.Rectangle bounds = gc.getBounds();
        if (this.windowMode==2)                                          
        {
            xsize=bounds.width;
            ysize=bounds.height;
            if (inifile.getTrueOrFalse("DOUBLE-SIZE"))
            {
                xsize/=2;
                ysize/=2;
            }
        }
        
        if (this.windowMode>0) 
        {                                                       // take the entire screen
            this.setLocation(bounds.x,bounds.y);
            java.awt.Dimension size=new java.awt.Dimension (bounds.width,bounds.height);
            this.setSize(size);                                 // make window as big as screen
            setUndecorated(true);
            try
            {
                java.awt.GraphicsDevice device = gc.getDevice();
                if (isMac)  
                {
                    MacUI.setFullScreen(this,device,11);
                }
                else if (isWindows)
                {
                    this.setExtendedState(javax.swing.JFrame.MAXIMIZED_BOTH); 
                }
                else if (device.isFullScreenSupported())
                {
                    device.setFullScreenWindow(this);
                }
                else
                    this.setExtendedState(javax.swing.JFrame.MAXIMIZED_BOTH); 
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }   
        else
            this.setLocation(this.inifile.getOptionAsInt("WDW-XPOS",0),this.inifile.getOptionAsInt("WDW-YPOS",0));
        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
   
        // fiddling about with the keyboard was moved to its own method, and is called from fromWindowOpened  (v. 1.40)
        setTitle("SMSQmulator v."+Localization.getVersion());
        this.setIconImage(getFDImage("images/smsqmulator2.png"));// fame at last...
        
        this.monitorPanel=new MonitorPanel (this.monitor,this.inifile);
                
        changeScreen (xsize,ysize,inifile.getOptionAsInt("SCREEN-MODE", 0));
        this.add(this.screen,java.awt.BorderLayout.CENTER);
        this.add(this.monitorPanel,java.awt.BorderLayout.PAGE_END);
        this.monitorPanel.setVisible(this.inifile.getTrueOrFalse("MONITOR-VISIBLE"));
       
        this.screen.setIsMac(isMac);
        this.screen.setIsWindows (isWindows);
       
        int ppause=inifile.getOptionAsInt("MOUSE-CLICK-DELAY", 0);
        this.screen.setMouseClickDelay(ppause);
        int accel=inifile.getOptionAsInt("MOUSEWHEEL-ACCEL", 0);
        if (accel>0 & accel<10)
            this.screen.setMousewheelAccel(accel);
        
        try
        {
            this.robot=new java.awt.Robot();                    // create the robot for mouse movement
            this.robot.setAutoDelay(0);
            this.robot.setAutoWaitForIdle(false);
        }
        catch (java.awt.AWTException e)
        {
            this.robot = null;                                  // if robot can't be created, there will be no pointer pos setting from within SMSQE
        }  
        
        javax.swing.JMenuBar menuBar = new javax.swing.JMenuBar();
        if (!this.inifile.getTrueOrFalse("MENUBAR-VISIBLE"))
            menuBar.setVisible(false);
        menuBar.add(new FileMenu(this.monitor,inifile));
        menuBar.add(new ConfigMenu(this.monitor,inifile,this,new Warnings(inifile),td));
        menuBar.add(new AboutMenu());
        this.setJMenuBar(menuBar);
        
        this.monitorPanel.getRegLogger().append("Use the small window below to type your commands.\nType 'h' for help.\n");
        
        int temp=this.inifile.getOptionAsInt("DIVIDER-LOCATION",0);
        if (temp!=0)
            this.monitorPanel.setDividerLocation(temp);
        this.pack();
        
        addListeners();
    }
    
    /**
     * Used when in Windows. If I can't stop the Windows LAF from opening a systems menu when ALT+SPACE
     * is hit, at least I can try to close that window again immediately.
     */
    private void doAltSpace(boolean shift)
    {
        java.awt.event.KeyEvent evt;
        if (shift)
            evt=new java.awt.event.KeyEvent(getRootPane(),java.awt.event.KeyEvent.VK_SPACE,System.currentTimeMillis(),
               java.awt.event.InputEvent.SHIFT_DOWN_MASK+java.awt.event.InputEvent.ALT_DOWN_MASK,java.awt.event.KeyEvent.VK_SPACE,' ');
        else 
            evt=new java.awt.event.KeyEvent(getRootPane(),java.awt.event.KeyEvent.VK_SPACE,System.currentTimeMillis(),
                java.awt.event.InputEvent.ALT_DOWN_MASK,java.awt.event.KeyEvent.VK_SPACE,' ');
        screen.screenPanelKeyTyped(evt);        // type in ALT+SPACE
        screen.requestFocus();
        if (this.robot != null)
        {
            this.robot.mousePress( java.awt.event.InputEvent.BUTTON1_DOWN_MASK);    // push left mousebutton (gets rid of menu)
        }
        screen.screenPanelKeyReleased(evt);     // now release alt+space keys again!!!!!!
        if (this.robot != null)
        {
            this.robot.mouseRelease(java.awt.event.InputEvent.BUTTON1_DOWN_MASK);  // release left mouse button again
        }
    }
     
    /**
     *  Fiddling about with the keyboard.
     */
    private void fiddleWithKeyboard()
    {
  
        // 1 - Stop tab as change focus key
        java.util.HashSet<java.awt.AWTKeyStroke> set = new java.util.HashSet<>
                 (java.awt.KeyboardFocusManager.getCurrentKeyboardFocusManager().getDefaultFocusTraversalKeys
                    (java.awt.KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS));
        set.clear();                                        // no more change focus keys!- this means that TAB is treated correctly
        java.awt.KeyboardFocusManager.getCurrentKeyboardFocusManager().setDefaultFocusTraversalKeys(java.awt.KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS, set);
        java.awt.KeyboardFocusManager.getCurrentKeyboardFocusManager().setDefaultFocusTraversalKeys(java.awt.KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS, set);
        
        // 2 - Stop F10 bringing up strange menus (under linux & windws)
        javax.swing.Action doNothing = new javax.swing.AbstractAction() 
        {
            @Override
            public void actionPerformed(java.awt.event.ActionEvent actionEvent) 
            {
              // System.out.println("ALT F10");//just do nothing
            }
        };
        javax.swing.JPanel tcontent = (javax.swing.JPanel) getContentPane();
        javax.swing.InputMap inputMap = tcontent.getInputMap(javax.swing.JComponent.WHEN_IN_FOCUSED_WINDOW);
        javax.swing.KeyStroke stroke = javax.swing.KeyStroke.getKeyStroke("F10");
        inputMap.put(stroke, "F10");
        tcontent.getActionMap().put("F10", doNothing);
       // tcontent.getActionMap().put("F10", null);  
  
                   /*    tests, leave for future reference
                    stroke = javax.swing.KeyStroke.getKeyStroke("alt F3");
                    inputMap = this.getRootPane().getInputMap(javax.swing.JComponent.WHEN_IN_FOCUSED_WINDOW);
                    inputMap.put(stroke, "alt_F3");
                    inputMap = this.getRootPane().getInputMap(javax.swing.JComponent.WHEN_FOCUSED);
                    inputMap.put(stroke, "alt_F3");
                    inputMap = this.getRootPane().getInputMap(javax.swing.JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
                    inputMap.put(stroke, "alt_F3");
                    this.getRootPane().getActionMap().put("alt_F3", doNothing);
                   */

        // 3 - stop Alt+Space bringing up a submenu in Windows (actually, I can't stop the submenu from opening, it but I close it again immediately)
        if (System.getProperty("os.name").toLowerCase().contains("windows"))
        {
            javax.swing.Action actionListenerAltSpace = new javax.swing.AbstractAction() 
            {
                @Override
                public void actionPerformed(java.awt.event.ActionEvent actionEvent) 
                {
                   doAltSpace(false);
                }
            };  
            javax.swing.Action actionListenerAltShiftSpace = new javax.swing.AbstractAction() 
            {
                @Override
                public void actionPerformed(java.awt.event.ActionEvent actionEvent) 
                {
                   doAltSpace(true);
                }
            };
            inputMap = this.getRootPane().getInputMap(javax.swing.JComponent.WHEN_IN_FOCUSED_WINDOW);   
            stroke = javax.swing.KeyStroke.getKeyStroke("alt SPACE");
            inputMap.put(stroke, "xxx");
            this.getRootPane().getActionMap().put("xxx", actionListenerAltSpace);
            
            stroke = javax.swing.KeyStroke.getKeyStroke("shift alt SPACE");
            inputMap.put(stroke, "xsp");
            this.getRootPane().getActionMap().put("xsp", actionListenerAltShiftSpace);
        }
    }
    
    /**
     * Change the language and thus the texts shown in the items, menu etc..
     * 
     * @param language 
     */
    public void setLanguage(int language)
    {
        Localization.setLanguage(language);
        javax.swing.JMenuBar menuBar = this.getJMenuBar();
        menuBar.removeAll(); 
        menuBar.invalidate();
        menuBar.add(new FileMenu(this.monitor,inifile));
        menuBar.add(new ConfigMenu(this.monitor,inifile,this,new Warnings(inifile),this.monitor.getTrapDispatcher()));
        menuBar.add(new AboutMenu());        
    }
    
    /************* Getters/ setters ************************/
    
    
    /**
     * Gets the screen currently used (not to be confused with the physical display this is shown on)
     * 
     * @return the screen currently used (not to be confused with the physical display on which this is shown)
     */
    public Screen getScreen()
    {
        return this.screen;
    }
    
    /**
     * Gets the GraphicsDevice currently used by this program.
     * 
     * @return 
     */
    public java.awt.GraphicsDevice [] getGds()
    {
        return this.gds;
    }
               
    /**
     * Gets the current window mode.
     * 
     * @return the current window mode (0:window  1:full size 2:special full size)
     */
    public int getWindowMode()
    {
        return this.windowMode;
    }
    
    /**
     * Gets the left monitor "wdw".
     * 
     * @return the <code>javax.swing.JTextArea</code> corresponds to the left wdw.
     */
    public javax.swing.JTextArea getDataLogger()
    {
        return this.monitorPanel.getDataLogger();
    } 
    
    /**
     * Gets the right monitor "wdw".
     * 
     * @return the <code>javax.swing.JTextArea</code> corresponds to the right wdw.
     */
    public javax.swing.JTextArea getRegLogger()
    {
        return this.monitorPanel.getRegLogger();
    }
    
    /**
     * Gets the command line "wdw".
     * 
     * @return the command line "wdw", a <code>javax.swing.JTextField</code>.
     */
    public javax.swing.JTextField getInputWindow()
    {
        return this.monitorPanel.getInputWindow();
    }
  
    /**
     * Sets the focus to the screen object.
     */
    public void setFocus()
    {
        if (this.screen!=null)
        {
            this.screen.requestFocus();
            this.screen.requestFocusInWindow();
        }
    }


 /*----------------------------------------- */
    /**
     * Make a new screen.
     * 
     * @param xsize         new x size
     * @param ysize         new y size
     * @param qlColourmode  new Ql colour mode (0,2,3)
     */
    private void changeScreen(int xsize,int ysize,int qlColourmode)   // xxxx keep
    {
        boolean vibrant=inifile.getTrueOrFalse("VIBRANT-COLOURS");
        boolean keyType = inifile.getTrueOrFalse("STANDARD_HOME_END_KEYS");
        boolean isDouble = inifile.getTrueOrFalse("DOUBLE-SIZE");
        if (this.screen!=null)
        {
       //     this.remove(this.monitorPanel);
            this.remove(this.screen); 
            this.screen=null;                                   // prepare for garbage collection
        }
        
        int mem = inifile.getOptionAsInt("MEM_SIZE", 32);
        mem *= smsqmulator.Types.ONE_MiB;
        switch (qlColourmode)
        {
            case 3:
                this.screen=new screens.Screen32(xsize,ysize,mem,vibrant,this.monitor,false,isDouble, this.windowMode, keyType);// make entirely new screen, mode 32
                break;  
            case 2:
                this.screen=new screens.Screen16(xsize,ysize,mem,vibrant,this.monitor,false,isDouble, this.windowMode, keyType);// make entirely new screen, mode 32
                break;
            default:
                if (xsize == 512 &&  ysize == 256)              // make a QL compatible screen at $20000
                    this.screen=new screens.Screen0(512,256,-1,this.monitor,false,isDouble, this.windowMode, keyType);// these are the addresses of the QL screen 
                else
                    this.screen=new screens.Screen0(xsize,ysize,0,this.monitor,false,isDouble, this.windowMode, keyType);// make entirely new screen
        }
        this.add(this.screen,java.awt.BorderLayout.CENTER);
  //      this.add(this.monitorPanel,java.awt.BorderLayout.PAGE_END);
        this.monitorPanel.setVisible(this.inifile.getTrueOrFalse("MONITOR-VISIBLE"));
        pack(); 
    }

    /**
     * Change the screen type/size/mode.
     * 
     * @param xsize         new X size
     * @param ysize         new Y size
     * @param qlColourmode  new colour mode (0, 2, or 3)
     */
    public void changeScreenAndReset(int xsize,int ysize,int qlColourmode)
    {
        this.inifile.writeIniFileNoError();                     // write new screen info
        changeScreen(xsize,ysize,qlColourmode);
        formMoved();                                            // make sure screen end positions are set                    
        this.monitor.reset();
        setFocus();
    }
  
    /**
     * Menu item action : Shows / hides the monitor window.
     * 
     * @param b true if monitor should be visible
     */
    public void monitorVisible(boolean b)
    {
        this.monitorPanel.setVisible(b);
        this.pack();
        ConfigData.setOptionAndSave("MONITOR-VISIBLE",b? "1":"0",this.inifile);
    }
    
    /**
     * Menu item action : Doubles/halves the window x and y sizes WITHOUT increasing resolution.
     */
    public void setDoubleSize()                                 // is called from ScreenConfigMenu
    {
        this.screen.changeDoubleSize();
        invalidate();
        pack();
        ConfigData.setOptionAndSave("DOUBLE-SIZE", this.screen.isDoubleSize()?"1":"0",this.inifile);
    }
    
    /**
     * Makes the menu bar visible/invisible.
     * Note: making it visible is done from within SMSQ/E, JVAMBAR command.
     *
     * @param state = true if menu bar is to be visible, false if not.
     */
    public void setMenuBarVisible(boolean state)
    {
        this.getJMenuBar().setVisible(state);
        pack();
        ConfigData.setOptionAndSave("MENUBAR-VISIBLE",  state ?"1":"0",this.inifile);
    }

  /******************************** Form opening / iconifying / de-iconifying ********************************/
    
    /**
     * Adds the mouse and window listeners .
     */
    private void addListeners()
    {
        addComponentListener(new java.awt.event.ComponentAdapter()
        {
            @Override
            public void componentMoved(java.awt.event.ComponentEvent evt)
            {
                 formMoved();
            }
            @Override
            public void componentResized(java.awt.event.ComponentEvent evt)
            {
                formMoved();
            }
        });
        
        addWindowListener(new java.awt.event.WindowAdapter()
        {
            @Override
            public void windowActivated(java.awt.event.WindowEvent evt)
            {
                formMoved();
                windowActivated=true;
            }
            @Override
            public void windowDeactivated(java.awt.event.WindowEvent evt)
            {
                windowActivated=false;
            }
            @Override
            public void windowDeiconified(java.awt.event.WindowEvent evt)
            {
                formWindowDeiconified();
            }
            @Override
            public void windowIconified(java.awt.event.WindowEvent evt)
            {
                formWindowIconified();
            }
            @Override
            public void windowOpened(java.awt.event.WindowEvent evt)
            {
                formWindowOpened();
            }
        });
        
        
    }
    
    /**
     * Sets the new screen coordinates when window has moved/resized.
     * 
     * @param evt ignored
     */
    private void formMoved() 
    {
       java.awt.Point p =this.getLocation();
        if (this.windowMode<1)
        {
            this.inifile.setOptionValue("WDW-XPOS",""+p.x);
            ConfigData.setOptionAndSave("WDW-YPOS",""+p.y,this.inifile);
        }
        this.screen.setEndPositions(p);
    }
    
    /**
     * When form opens, load rom if possible, shift focus to screen window and start emulation.
     * 
     */
    private void formWindowOpened() 
    {   
        ConfigData.setOptions(this.inifile, this.monitor);// gets various options from the inifile and sets them.
        this.monitor.reset();
        formMoved();
        fiddleWithKeyboard();
        setFocus();
    }
    
    /**
     * 
     */
    private void formWindowIconified() 
    {
        this.monitor.removeAllKeyrows();
        this.screen.resetMouseButtons();
        if (this.inifile.getTrueOrFalse("SUSPEND-WHEN-ICONIFIED"))
        {
            try
            {
                Thread.sleep(500);      // kludge for windows, linux doesn't need the sleep. Windows does, else deiconified is never called.
            }
            catch (InterruptedException p)
            {
                //nop
            }
            this.monitor.suspendExecution();
        }
    }

    /**
     * 
     */
    private void formWindowDeiconified() 
    {
         if (this.inifile.getTrueOrFalse("SUSPEND-WHEN-ICONIFIED"))       
            this.monitor.resumeExecution();
       
        java.awt.Point p =this.screen.getMyMousePosition();
        if (this.screen.isDoubleSize())
        {
           p.x/=2;
           p.y/=2;
        }
        setMousePosition (p.x,p.y);  
        this.monitor.inputMouse(0,0,p.x,p.y);
        setFocus();
    }
    


  /******************************** Misc ******************************/
  
   
    /**
     * Pops the window up if iconified, or flashes the taskbar.
     */
    public void deIconify()
    {
        if ((this.getExtendedState()&ICONIFIED)!=0)
        {
            if (inifile.getTrueOrFalse("POPUP-ACTION"))
            {
                java.awt.Image ic = getFDImage("images/smsqmulatorinv.png");
                TBFlasher mflasher=new TBFlasher(this,ic);
                mflasher.execute();
            }
            else
            {
                this.setExtendedState(NORMAL);
            }
        }
    }
    
    /**
     * Sets a new mouse position if that has changed from within SMSQ/E.
     * 
     * @param x the new x position, relative to the screen object.
     * @param y same for y.
     */
    public void setMousePosition(int x,int y)
    {
        if (!( this.windowActivated & this.screen.mouseInScreen()))
            return;
        if (this.robot!=null)         
        {
            java.awt.Point p=this.screen.getLocationOnScreen(); 
            if (this.screen.isDoubleSize())
                this.robot.mouseMove(x*2 +p.x, y*2+p.y);
            else
                this.robot.mouseMove(x+p.x, y+p.y);
        }
    }
     
    /**
     * Iconifies wdw from SMSQE command line.
     */
    public void iconify()
    {
        this.setExtendedState(ICONIFIED);
    }
   
    
    /**
     * Returns an Image or null.
     * 
     * @param filename where to get the image from.
     * 
     * @return the Image or null.
     */
    public static java.awt.Image getFDImage(String filename)
    {
        java.net.URL imgURL = MonitorGui.class.getResource(filename);
        if (imgURL != null)
            return new javax.swing.ImageIcon(imgURL).getImage();
        else
            return null;
    }
    
    /**
     * Gets the monitor.
     * 
     * @return the monitor.
     */
    public final Monitor getMonitor()
    {
        return this.monitor;
    }
}

    
    /*    
     * unsuccesful attempts to stop alt + space under windows
        stroke = javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_SPACE,java.awt.event.InputEvent.ALT_MASK);
        inputMap.put(stroke, "SPACE");
        tcontent.getActionMap().put("SPACE", actionListener);
        */
        
   /*
        javax.swing.InputMap inputMap = new javax.swing.InputMap();
        rootPane.setInputMap(1,null);
        * 
        *
        // try to stop altkey escaping the wdw, doesn't work (yet?)
        this.addFocusListener(new java.awt.event.FocusListener() {
            private final java.awt.KeyEventDispatcher altDisabler = new java.awt.KeyEventDispatcher() 
            {
                @Override
                public boolean dispatchKeyEvent(java.awt.event.KeyEvent e) 
                {
                    return e.getKeyCode() == 18;
                }
            };

            @Override
            public void focusGained(java.awt.event.FocusEvent e) 
            {
                java.awt.KeyboardFocusManager.getCurrentKeyboardFocusManager().addKeyEventDispatcher(altDisabler);
            }

            @Override
            public void focusLost(java.awt.event.FocusEvent e) 
            {
                java.awt.KeyboardFocusManager.getCurrentKeyboardFocusManager().removeKeyEventDispatcher(altDisabler);
            }
        });
      */
     //    java.awt.KeyboardFocusManager.getCurrentKeyboardFocusManager().addKeyEventDispatcher(this);
     //    java.awt.KeyboardFocusManager.getCurrentKeyboardFocusManager().
    
    /* 
    @Override // trying to trap the alt + space key on windows
    public boolean dispatchKeyEvent(java.awt.event.KeyEvent e) 
    {
        //  if ( e.getKeyCode() == 18)//alt key
        if ( e.getKeyCode() == 32 && e.getModifiers()==java.awt.event.InputEvent.ALT_MASK)//alt key
        {
            e.consume();
            return true;
        }
        else
            return false;
    }
    */
       /*
        // 2 - Stop F10 bringing up strange menus (under linux) - this doesn't work
        javax.swing.Action actionListener = new javax.swing.AbstractAction() 
        {
            @Override
            public void actionPerformed(java.awt.event.ActionEvent actionEvent) 
            {
              // System.out.println("ALT F3");//just do nothing
            }
        };

        javax.swing.JPanel tcontent = (javax.swing.JPanel) getContentPane();
        javax.swing.InputMap inputMap = tcontent.getInputMap(javax.swing.JComponent.WHEN_IN_FOCUSED_WINDOW);
        
        javax.swing.KeyStroke stroke = javax.swing.KeyStroke.getKeyStroke("F10");
        inputMap.put(stroke, "F10");
        tcontent.getActionMap().put("F10", actionListener);
        
        // 3 - stop Alt+Space bringing up a submenu in windows -this doesn't work!!!!!
        stroke = javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_SPACE,java.awt.event.InputEvent.ALT_MASK);
        inputMap.put(stroke, "SPACE");
        tcontent.getActionMap().put("SPACE", actionListener);
        */
