/*
 * Copyright (c) W. Lenerz 2023.
 * This code is subject to the GPL, version 3 or later.
 */
package ui;


/**
 * The screen section of the config menu in the menu bar, including item action routines.
 * 
 * @author and copyright Wolfgang Lenerz
 * @version 1.01 change string in "changesize" to make it all html, else java 21 has a problem (it only displays the html part).
 * @version 1.00 initial version
 */
public class ScreenMenu extends javax.swing.JMenu
{
    private final smsqmulator.Monitor monitor;
    private final inifile.IniFile inifile;
    private final MonitorGui mgui;
    
    public ScreenMenu (smsqmulator.Monitor pmonitor,inifile.IniFile pinifile,MonitorGui mguip)
    {
        this.monitor=pmonitor;
        this.inifile=pinifile;
        this.mgui=mguip;
        initMenu();
    }
    private void initMenu()
    {
        setText(Localization.Texts[157]);
        javax.swing.JMenuItem mit;
        javax.swing.JCheckBoxMenuItem cbox;
        javax.swing.JMenu menu;
        javax.swing.ButtonGroup buttonGroup;
        javax.swing.JRadioButtonMenuItem rbutton;
        
        mit = new javax.swing.JMenuItem();                      // screen size
        mit.setText(Localization.Texts[16]);                     
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            changeSize();
        });
        this.add(mit);
        
        int smode= this.inifile.getOptionAsInt("SCREEN-MODE",0); 
        buttonGroup = new javax.swing.ButtonGroup() ;
        menu=new  javax.swing.JMenu();
        menu.setText(Localization.Texts[17]);                   // screen  mode
            rbutton = new javax.swing.JRadioButtonMenuItem();   // 16 bit
            buttonGroup.add(rbutton);
            rbutton.setText(Localization.Texts[24]);
            rbutton.addActionListener((java.awt.event.ActionEvent evt) ->
            {
                  setScreenModeForNextRestart (3);
            });
            if (smode==3)
                rbutton.setSelected(true);
            menu.add(rbutton);
            rbutton = new javax.swing.JRadioButtonMenuItem();   // 8 bit
            buttonGroup.add(rbutton);
            rbutton.setText(Localization.Texts[107]);
            rbutton.addActionListener((java.awt.event.ActionEvent evt) ->
            {
                  setScreenModeForNextRestart (2);
            });
            if (smode==2)
                rbutton.setSelected(true);
            menu.add(rbutton);
            rbutton = new javax.swing.JRadioButtonMenuItem();   // ql colours
            buttonGroup.add(rbutton);
            rbutton.setText(Localization.Texts[25]);
            rbutton.addActionListener((java.awt.event.ActionEvent evt) ->
            {
                  setScreenModeForNextRestart (0);
            });
            if (smode==0)
                rbutton.setSelected(true);
            menu.add(rbutton);
            
        this.add(menu);
        
        int windowMode = this.mgui.getWindowMode();
      
        buttonGroup = new javax.swing.ButtonGroup() ;
        menu=new  javax.swing.JMenu();
        menu.setText(Localization.Texts[134]);                  // window mode
        rbutton = new javax.swing.JRadioButtonMenuItem();       // window
            buttonGroup.add(rbutton);
            rbutton.setText(Localization.Texts[135]);
            rbutton.addActionListener((java.awt.event.ActionEvent evt) ->
            {
                 ConfigData.setOptionAndSave("Window-MODE", "0", this.inifile);
            });
            if (windowMode==0)
                rbutton.setSelected(true);
            menu.add(rbutton);
            rbutton = new javax.swing.JRadioButtonMenuItem();   // full size
            buttonGroup.add(rbutton);
            rbutton.setText(Localization.Texts[136]);
            rbutton.addActionListener((java.awt.event.ActionEvent evt) ->
            {
                 ConfigData.setOptionAndSave("Window-MODE", "1", this.inifile);
            });
            if (windowMode==1)
                rbutton.setSelected(true);
            menu.add(rbutton);
            rbutton = new javax.swing.JRadioButtonMenuItem();   // special full size
            buttonGroup.add(rbutton);
            rbutton.setText(Localization.Texts[137]);
            rbutton.addActionListener((java.awt.event.ActionEvent evt) ->
            {
                 ConfigData.setOptionAndSave("Window-MODE", "2", this.inifile);
            });
            if (windowMode==2)
                rbutton.setSelected(true);
            menu.add(rbutton);
        this.add(menu);
        
        cbox=new javax.swing.JCheckBoxMenuItem();
        cbox.setText(Localization.Texts[109]);                  // vibrant colours?
        cbox.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            javax.swing.JCheckBoxMenuItem cb = (javax.swing.JCheckBoxMenuItem) evt.getSource();
            ConfigData.setOptionAndSave("VIBRANT-COLOURS", cb.isSelected() ?"1":"0",this.inifile);
        });
        cbox.setSelected(inifile.getTrueOrFalse("VIBRANT-COLOURS"));
        this.add(cbox);
        
        mit = new javax.swing.JMenuItem();                      // double size 
        if (this.mgui.getScreen().isDoubleSize())
            mit.setText(Localization.Texts[97]);
        else
            mit.setText(Localization.Texts[96]);               
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            this.mgui.setDoubleSize();
            javax.swing.JMenuItem mitem = (javax.swing.JMenuItem)evt.getSource();
            if (this.mgui.getScreen().isDoubleSize())
                mitem.setText(Localization.Texts[97]);
            else
                mitem.setText(Localization.Texts[96]);   
        });
        this.add(mit);
        
        mit = new javax.swing.JMenuItem();                      // update rate
        mit.setText(Localization.Texts[129]);          
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            setUpdateRate();
        });
        this.add(mit);
        
        cbox=new javax.swing.JCheckBoxMenuItem();
        cbox.setText(Localization.Texts[142]);                  // allow ql screen emu
        cbox.setSelected(this.inifile.getTrueOrFalse("QL-SCREEN-EMULATION"));
        cbox.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            javax.swing.JCheckBoxMenuItem cb = (javax.swing.JCheckBoxMenuItem) evt.getSource();
            ConfigData.setOptionAndSave("QL-SCREEN-EMULATION", cb.isSelected() ?"1":"0",this.inifile);
        });
        this.add(cbox);
        
        mit = new javax.swing.JMenuItem();  
        mit.setText(Localization.Texts[138]);          
        mit.addActionListener((java.awt.event.ActionEvent evt) ->
        {
            MultiMonitorDialog mmd= new MultiMonitorDialog(this.mgui,true,this.mgui.getGds(),this.inifile.getOptionAsInt("SCREEN-NUMBER",0));
            int choice=mmd.getChoice();
            if (choice!=-1)
            {
                 ConfigData.setOptionAndSave("SCREEN-NUMBER", ""+choice, this.inifile);
            }
        });
        this.add(mit);
    }
 
    /** 
     * Menu item action : This prompts for a new screen size, changes the screen size and restarts the emulation.
     * 
     * @param evt ignored.
     */
    private void changeSize() 
    {                                                   
        screens.Screen screen =  this.mgui.getScreen();
        java.awt.Dimension size=screen.getImageSize();
        String s = (String)javax.swing.JOptionPane.showInputDialog
            (
                    this,
                    "<html>"+ Localization.Texts[40]+"<br>"+
                    Localization.Texts[41]+" '512 x 256' "+Localization.Texts[42]+" '1024 x 512'<br>"+
                    Localization.Texts[98]+"<br>"+
                    Localization.Texts[43]+size.width+" x "+size.height+"<br>"+
                    Localization.Texts[95]+"<br>"+
                    "<font color=red>"+Localization.Texts[29]+"</font></html>",
                    Localization.Texts[44],
                    javax.swing.JOptionPane.PLAIN_MESSAGE,
                    null,
                    null,
                    null
            );                                                  // get user input
        if (s==null || s.isEmpty())
            return;                                             // nothing, just leave
        try
        { 
            String []options=s.split("x");                      // input size as a x b
            if (options.length!=2)
            {
                options=s.split("\\*");                         // input size as a * b
                if (options.length!=2)
                {
                    throw new java.util.InputMismatchException();// wrong parameters
                }
            }
            int xsize=Integer.parseInt(options[0].trim());
            int ysize=Integer.parseInt(options[1].trim());
            if (xsize<512)
                xsize=512;
            if (ysize<256)
                ysize=256;                                      // screen has 512 x 256 as minimum size
            xsize=xsize+  ((xsize % 8)==0?  0 : (8-(xsize % 8)));// make xsize a multiple of 8
            this.inifile.setOptionValue("WDW_XSIZE", ""+xsize);
            this.inifile.setOptionValue("WDW_YSIZE", ""+ysize);
            this.mgui.changeScreenAndReset (xsize,ysize,screen.getColourMode());
        } 
        catch (java.util.InputMismatchException | NumberFormatException e)
        {
            smsqmulator.Helper.reportError(Localization.Texts[45], Localization.Texts[40], this.mgui);
        }
    }
    
    /**
     * Sets the screen mode for the next restart of the emulation.
     * 
     * @param choice 0 for QL mode, 2 for 8bit aurora mode, 3 for 16 bit colours.
     */
    private void setScreenModeForNextRestart(int choice)
    {
        int newMode;
        switch (choice)                                         // use this as a simple check : any value that isn't 0,2,3 is set to 0
        {
            case 0:
            case 2:
            case 3:
                newMode=choice;
                break; 
            default:
                newMode = 0;
                break;
        }
        this.inifile.setOptionValue("SCREEN-MODE", ""+newMode);
        screens.Screen screen = this.mgui.getScreen();
        this.mgui.changeScreenAndReset(screen.getXSize(),screen.getYSize(), newMode);
    }
    
    /**
     * Get the new screen update rate and set it.
     */
    private void setUpdateRate()
    {
        int screenUpdateRate =  this.inifile.getOptionAsInt("SCREEN-UPDATE-RATE", 50);
        String s = (String)javax.swing.JOptionPane.showInputDialog(this, Localization.Texts[130]+"\n"+Localization.Texts[131]+
                screenUpdateRate,
                Localization.Texts[129],javax.swing.JOptionPane.PLAIN_MESSAGE, null, null, ""+screenUpdateRate);// get update rate now
        if ((s != null) && (!s.isEmpty()))
        {
            try
            {
                screenUpdateRate=Integer.parseInt(s);  
                if (screenUpdateRate<0)
                    screenUpdateRate=0;
                if (screenUpdateRate>65535)
                    screenUpdateRate=65535;
                ConfigData.setOptionAndSave("SCREEN-UPDATE-RATE", s, this.inifile);  
                this.monitor.setScreenUpdateInterval(screenUpdateRate);
            }
            catch (Exception e)
            {
                javax.swing.JOptionPane.showMessageDialog(this, s+Localization.Texts[31], Localization.Texts[30],javax.swing.JOptionPane.ERROR_MESSAGE);
            }
        }
    }
}