/*
 * Copyright (c) W. Lenerz 2023.
 * This code is subject to the GPL, version 3 or later.
 */
package ui;;

/**
 * The dialog to assign machine names for serial ports to the SMSQE serial ports.
 * It contains one row with 3 columns per serial port. 
 * 
 * @author and copyright Wolfgang Lenerz
 * @version  1.00 
 */
public class SerAssignmentsDialog extends javax.swing.JDialog
{
    private final int size;                                        // max number of serial ports
    public static final String SER_SEP=" ( -> ";
    private boolean acceptChanges=false;                            // do not accept changes
    private final javax.swing.JTextField[]tf1;                      // textfields for new names  
    private final java.awt.Frame parentt;
    
    /**
     * Create the dialog.
     * 
     * @param parent    standard jdialog param   
     * @param modal     standard jdialog param   
     * @param serNames  array with serial port names as they are configured and contained in the ini file. Whilst each element of the ayyar 
     *                  may be empty or null, the array itself must NOT be null or empty. THIS IS NOT CHECKED HERE!
     */
    public  SerAssignmentsDialog(java.awt.Frame parent, boolean modal,String []serNames)
    {
        super(parent, modal); 
        this.parentt=parent;
        this.size=serNames.length;
        this.tf1=new javax.swing.JTextField[this.size]; // textfields for new names  
        initComponents(serNames);
        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
    }
    
  
     /**
     * Accept or cancel buttons pressed.
     * If accept : the new options are copied to the old array.
     * 
     * @param state =<code>true</code> if OK pressed, else false.
     */
    private void ButtonsActioned(boolean state)
    {
        this.acceptChanges=state;                           // are changes accepted or not?
        super.setVisible(false);
    }
    
    
    /**
     * This obviously does NOT show the dialog. Use getPorts() for this.
     */
    public void setVisible()
    {
    }
    
    /**
     * Returns the results of the user's changes, or NULL
     * 
     * @return string array with changes made by user, or null if user left with cancel button or ESC (meaning no change).
     */
    public final String [] getPorts()
    {
        super.setVisible(true);
        if (!this.acceptChanges)
            return null;
        else
        {
            String[]result = new String[this.size];
            for (int i = 0;i<this.size; i++)
            {
                String s = this.tf1[i].getText();
                int place = s.indexOf(SER_SEP);
                if (place!=-1)
                    s=s.substring(0,place).trim();
                result[i]=s;
            }
            return result;
        }
    }
 
    private void initComponents(String[] serNames)
    {
        this.setTitle(Localization.Texts[156]);
        java.awt.Container contentPane = this.getContentPane();
        contentPane.setLayout(new javax.swing.BoxLayout(contentPane,1));
        javax.swing.JPanel contentPanel = new javax.swing.JPanel();
        contentPanel.setLayout(new javax.swing.SpringLayout());
        java.awt.Dimension tfSize;
        javax.swing.JButton button;
        java.util.List<drivers.ser.SerPortDefinition> ports = drivers.ser.SerDriver.getSerialPorts();
        
        for (int i=0;i<this.size;i++)                               // build the textfields + buttons next to them
        {
            contentPanel.add(new javax.swing.JLabel("SER"+(i+1)));
            String name;
            name = serNames[i];
            if (name==null)
                name="";
            this.tf1[i]=new javax.swing.JTextField(name);// create the textfield with the right parameters
            tfSize=this.tf1[i].getPreferredSize();
            if (tfSize.width<200)
            {
                tfSize.width=200;
                this.tf1[i].setPreferredSize(tfSize);            
            }
            contentPanel.add(this.tf1[i]);
            contentPanel.add(makeComboBox(ports, this.tf1[i]));
        }
        
        contentPanel.add(new javax.swing.JLabel(""));           // bogus component to fill the last row
        
        button=new javax.swing.JButton (Localization.Texts[46]);// cancel button...
        button.addActionListener ((java.awt.event.ActionEvent evt) ->
        {
            ButtonsActioned(false);                             // ... with an action routine (same as for OK button!)
        });
        contentPanel.add(button); 
        
        button=new javax.swing.JButton ("OK");                  // OK button...
        button.addActionListener ((java.awt.event.ActionEvent evt) ->
        {
            ButtonsActioned(true);                              // with an action routine                 
        });
        contentPanel.add(button);
        
        smsqmulator.SpringUtilities.makeCompactGrid(contentPanel,           // use the spring utilities to put everything in a grid
                                 this.size+1, 3,                // rows, cols
                                 4, 4,                          // initialX, initialY
                                 4, 4);                         // xPad, yPad
        contentPane.add(contentPanel);
        
        java.awt.event.ActionListener actionListener = (java.awt.event.ActionEvent actionEvent) ->
        {
            ButtonsActioned(false);                             // action routine for esc keystroke in wdw
        } 
        ;
        javax.swing.KeyStroke stroke = javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_ESCAPE, 0);
        this.rootPane.registerKeyboardAction(actionListener, stroke, javax.swing.JComponent.WHEN_IN_FOCUSED_WINDOW);
        pack();
    }  
    
    /**
     * I stupidly need to make a new combobox for every row.
     * 
     * @param ports the list with serport definitions
     * 
     * @return the newlay made javax.swing.JComboBox
     */
    private javax.swing.JComboBox <String> makeComboBox (java.util.List<drivers.ser.SerPortDefinition> ports, javax.swing.JTextField tf)
    {
        javax.swing.JComboBox <String> comboBox = new javax.swing.JComboBox<>();
        for (drivers.ser.SerPortDefinition port:ports)
        {
            String s = port.OSname+SER_SEP+port.name+")";
       //     String s = port.OSname+SER_SEP+port.name+" "+port.description+")";
            comboBox.addItem(s);
        }
        comboBox.addActionListener ((java.awt.event. ActionEvent e) -> 
        {
            String s = (String)comboBox.getSelectedItem();
            if (s!=null)
            {
                int place = s.indexOf(SER_SEP);
                if (place!=-1)
                    s=s.substring(0,place).trim();
                tf.setText(s);
            }
        });
        return comboBox;
    }
}
